--[[ DEV NOTES
TEXTURES\SPELLACTIVATIONOVERLAYS\MONK_OX.BLP
TEXTURES\SPELLACTIVATIONOVERLAYS\BLOOD_BOIL.BLP (needs to be rotated 90 degree clockwise
TEXTURES\SPELLACTIVATIONOVERLAYS\GENERIC_ARC_04.BLP (PG indicator)
]]

-- Locals
local loaded
local tigerPowerUp
local elapsedTime = 0
local outOfCombatTime

-- Functions

local function loadOrCreateOptions()
	if NorthernAleOptions == nil then
		print("Options reset")
		NorthernAleOptions = {
			-- pgFrameOn = true,
			tigerPowerFrameOn = true,
			shuffleFrameOn = true,
			elusiveBrewButtonOn = true,
			procWatchOn = true,
			fauxStaggerOn = false,
		}
	end
end

local function NAOverlaysetTexture(frame, side, texture, sizeScale, startHidden, ...)
	local offsetX, offsetY, startHiden = ...
	if not shortScale then shortScale = 1 end
	local longSide = 256 * sizeScale;
	local shortSide = 128 * sizeScale; -- * shortScale;
	if not offsetX then offsetX = 0 end
	if not offsetY then offsetY = 0 end
	
	frame:ClearAllPoints()
	
	if side == "L" then
		local texLeft, texRight, texTop, texBottom = 0, 1, 0, 1
		frame:SetPoint("RIGHT", NorthernAle.northernAleOverlayFrame, "LEFT", offsetX, offsetY);
		frame:SetTexCoord(texLeft, texRight, texTop, texBottom)
	elseif side == "R" then
		offsetX = offsetX * -1
		local texLeft, texRight, texTop, texBottom = 1, 0, 0, 1
		frame:SetPoint("LEFT", NorthernAle.northernAleOverlayFrame, "RIGHT", offsetX, offsetY);
		frame:SetTexCoord(texLeft, texRight, texTop, texBottom)
	end
	frame:SetTexture(texture)
	-- print("created " .. frame:GetName() .. "'s " ..side .. " side texture")
	if startHidden then
		-- print(frame:GetName() .. " is hidden")
		frame:Hide()
	else
		frame:Show()
	end
end


-- Updaters

local function UpdateShuffle()
	--print("Shuffle Update entered")
	if UnitBuff("player", "Shuffle") then
		--print("Shuffle buff found")
		local n,_,icon,count,_,dur,expTime,_,_=UnitBuff("player", "Shuffle")
		local timeLeft = expTime - GetTime()
		if timeLeft < 0 then timeLeft = 0 end
		-- print("Shuffle Updating; duration left = " .. timeLeft)
		if tonumber(timeLeft) > 30 then timeLeft = "30" end
		if tonumber(timeLeft) <= 6 then
			NorthernAle.shufflestatusFrame:SetStatusBarColor(1, 0.04, 0.02, 1)
		elseif tonumber(timeLeft) > 5 then
			NorthernAle.shufflestatusFrame:SetStatusBarColor(0.33, 0.54, 0.52, 1)
		end
		NorthernAle.shufflestatusFrame:SetValue(tostring(timeLeft))
		NorthernAle.shufflestatusFrame:Show()
	else
		NorthernAle.shufflestatusFrame:SetValue("0")
	end
end

local function UpdateTigerPower()
	if UnitBuff("player", "Tiger Power") and NorthernAleOptions.tigerPowerFrameOn then
		NorthernAle.leftTigerPowerImage:SetVertexColor(0.22, 0.76, 0.11, 1)
		NorthernAle.rightTigerPowerImage:SetVertexColor(0.22, 0.76, 0.11, 1) -- (57, 194, 29, 1)
		NorthernAle.leftTigerPowerImage:Show()
		NorthernAle.rightTigerPowerImage:Show()
		tigerPowerUp = true
	else
		NorthernAle.leftTigerPowerImage:Hide()
		NorthernAle.rightTigerPowerImage:Hide()
		tigerPowerUp = false
	end
end

-- Load Functionality

local function loadNorthernAle()
	local longSide = 256 * 0.8;
	NorthernAle.northernAleOverlayFrame:SetSize(longSide, longSide)
	
	NorthernAle.shufflestatusFrame:SetStatusBarTexture("TEXTURES\\SPELLACTIVATIONOVERLAYS\\GENERICTOP_01")
	NorthernAle.shufflestatusFrame:SetStatusBarColor(0.33, 0.54, 0.52, 1)
	if NorthernAleOptions.shuffleFrameOn then UpdateShuffle() end
	
	-- create PS_PG textures
	local TigerPowerImageFrames = {
		[1] = {NorthernAle.leftTigerPowerImage, "L",},
		[2] = {NorthernAle.rightTigerPowerImage, "R",},
	}
	for idx = 1, #TigerPowerImageFrames do
		NAOverlaysetTexture(TigerPowerImageFrames[idx][1], TigerPowerImageFrames[idx][2], "TEXTURES\\SPELLACTIVATIONOVERLAYS\\GENERICARC_04", 0.4, 40, false)
	end
	if NorthernAleOptions.tigerPowerFrameOn then UpdateTigerPower() end
		
	if NorthernAleOptions.fauxStaggerOn then
		-- print("option works")
		

		-- create faux stagger overlay
		local stagtests = {
			[1] = {NorthernAle.lstagtest, "L",},
			[2] = {NorthernAle.rstagtest, "R",},
		}
		
		for idx = 1, #stagtests do
			-- print(stagtests[idx][1]:GetName(), stagtests[idx][2])
			NAOverlaysetTexture(stagtests[idx][1], stagtests[idx][2], "TEXTURES\\SPELLACTIVATIONOVERLAYS\\MONK_OX", 0.8, true)
		end
		NorthernAle.overlaysInUse = {};
		NorthernAle.unusedOverlays = {};
		-- NorthernAle.lstagtest:Show()
		-- NorthernAle.rstagtest:Show()
	else
		-- print(NorthernAle.NorthernAleOptions.fauxStaggerOn)
	end
	
	NorthernAle:RegisterEvent("UNIT_AURA")
	NorthernAle:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	NorthernAle:RegisterEvent("PLAYER_REGEN_ENABLED")
	loaded = true
end

local function unloadNorthernAle()
	NorthernAle:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	NorthernAle:UnregisterEvent("PLAYER_REGEN_ENABLED")
	NorthernAle.UnregisterEvent("UNIT_AURA")
	
	loaded = false
end

local function checkClassAndSpec()
	local _, playerClass = UnitClass("player")
	if playerClass == "MONK" then
		NorthernAle:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
		if GetSpecialization() == 1 then
			if not loaded then
				print("*BURP!* (Northern Ale Enabled)")
				loadNorthernAle()
			end
		end
	else
		if loaded then
			print("BrewMaster's keg not detected: disabling Northern Ale")
			unloadNorthernAle()
		end
	end
end


-- from SpellActivationOverlay.lua
local complexLocationTable = {
	["RIGHT (FLIPPED)"] = {
		RIGHT = {	hFlip = true },
	},
	["BOTTOM (FLIPPED)"] = {
		BOTTOM = { vFlip = true },
	},
	["LEFT + RIGHT (FLIPPED)"] = {
		LEFT = {},
		RIGHT = { hFlip = true },
	},
	["TOP + BOTTOM (FLIPPED)"] = {
		TOP = {},
		BOTTOM = { vFlip = true },
	},
}

-- Beta Mode testing

local function fauxEAB()
	
end

-- Script handlers

function NorthernAle_OnLoad()
	NorthernAle:RegisterEvent("ADDON_LOADED")
end

function NorthernAle_OnEvent(self, event, ...)
	arg1 = ...
	if event == "ADDON_LOADED" and arg1 == "NorthernAle" then
		loadOrCreateOptions()
		checkClassAndSpec()
	end
	if event == "ACTIVE_TALENT_GROUP_CHANGED" then --or event == "PLAYER_ALIVE" then
		checkClassAndSpec()
	end
	if loaded then
		if event == "UNIT_AURA" then
			if NorthernAleOptions.shuffleFrameOn then UpdateShuffle() end
			if NorthernAleOptions.tigerPowerFrameOn then UpdateTigerPower() end
		elseif event == "PLAYER_REGEN_ENABLED" then
			outOfCombatTime = GetTime()
		end
	end
end

-- OnUpdate

function NorthernAle_OnUpdate(self, elapsed)
	if loaded then
		if outOfCombatTime and tigerPowerUp then
			local currentTime = GetTime()
			if (currentTime - outOfCombatTime) > 5 then
				print("Left Combat; hiding NA Frames.")
				NorthernAle.leftTigerPowerImage:Hide()
				NorthernAle.rightTigerPowerImage:Hide()
				outOfCombatTime = null
			end
		end
	
		elapsedTime = elapsedTime + elapsed
		if elapsedTime >= 0.1 then -- set back to 0.1 later
			if NorthernAleOptions.shuffleFrameOn then UpdateShuffle() end
			if NorthernAleOptions.tigerPowerFrameOn then UpdateTigerPower() end
			elapsedTime=0
		end
	end
end

-- Debug tools
local function GatherBuffData()
	local i = 1;

	--[[
	while UnitBuff("player", i) do
		local name, rank, iconTexture, count, debuffType, duration, expirationTime, one, two, three, four, spellID = UnitBuff("player", i)
		print("Buff " .. i .. ":")
		print("Name: " .. name)
		print("icon: ".. iconTexture)
		print("count: " .. count)
		print("duration in seconds: " .. duration)
		if spellID then
			print("Spell ID: " .. spellID)
		end
	end ]]
	
	
	for i=1,40 do 
		local n,_,icon,count,_,dur,x,_,_=UnitBuff("player",i);
		if n then
			print(i.."="..n..", "..icon..", "..dur.." seconds left.")
		end 
	end
	
end

-- Slash Commands
SLASH_NRTHRNALE1 = "/na"
SlashCmdList["NRTHRNALE"] = function(msg)
	if string.lower(msg) == "staghide" then
		NorthernAle.lstagtest:Hide()
		NorthernAle.rstagtest:Hide()
		NorthernAleOptions.fauxStaggerOn = false
	elseif string.lower(msg) == "stagshow" then
		NorthernAle.lstagtest:Show()
		NorthernAle.rstagtest:Show()
		NorthernAleOptions.fauxStaggerOn = true
	elseif string.lower(msg) == "shuffleon" then
		NorthernAle.shufflestatusFrame:Show()
		NorthernAleOptions.shuffleFrameOn = true
	elseif string.lower(msg) == "shuffleoff" then
		NorthernAle.shufflestatusFrame:Hide()
		NorthernAleOptions.shuffleFrameOn = false
	elseif string.lower(msg) == "buffdata" then
		GatherBuffData()
	elseif string.lower(msg) == "hide" then
		NorthernAle:Hide()
	elseif string.lower(msg) == "show" then
		NorthernAle:Show()
	elseif string.lower(msg) == "tigerpoweron" then
		NorthernAle.leftTigerPowerImage:Show()
		NorthernAle.rightTigerPowerImage:Show()
		NorthernAleOptions.tigerPowerFrameOn = true
	elseif string.lower(msg) == "tigerpoweroff" then
		NorthernAle.leftTigerPowerImage:Hide()
		NorthernAle.rightTigerPowerImage:Hide()
		NorthernAleOptions.tigerPowerFrameOn = false
	end
end